@@ -6,7 +6,7 @@ import sys
 import types
 
 import cherrypy
-from cherrypy._cpcompat import IncompleteRead, itervalues, ntob
+from cherrypy._cpcompat import IncompleteRead, itervalues, ntob, ntou
 from cherrypy import _cptools, tools
 from cherrypy.lib import httputil, static
 from cherrypy.test._test_decorators import ExposeExamples
@@ -147,6 +147,10 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             def url_with_quote(self):
                 raise cherrypy.HTTPRedirect("/some\"url/that'we/want")
 
+            def url_with_unicode(self):
+                raise cherrypy.HTTPRedirect(ntou("\u0442\u0435\u0441\u0442",
+                                                 'escape'))
+
         def login_redir():
             if not getattr(cherrypy.request, "login", None):
                 raise cherrypy.InternalRedirect("/internalredirect/login")
@@ -403,7 +407,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertStatus(('302 Found', '303 See Other'))
 
         # check injection protection
-        # See https://bitbucket.org/cherrypy/cherrypy/issue/1003
+        # See https://github.com/cherrypy/cherrypy/issues/1003
         self.getPage(
             "/redirect/custom?"
             "code=303&url=/foobar/%0d%0aSet-Cookie:%20somecookie=someval")
@@ -431,6 +435,12 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertStatus(303)
         assertValidXHTML()
 
+        # check redirect to url containing unicode characters.
+        self.getPage("/redirect/url_with_unicode")
+        self.assertStatus(303)
+        loc = self.assertHeader('Location')
+        assert loc.endswith("\xd1\x82\xd0\xb5\xd1\x81\xd1\x82")
+
     def test_InternalRedirect(self):
         # InternalRedirect
         self.getPage("/internalredirect/")
@@ -551,7 +561,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
     def skip_if_bad_cookies(self):
         """
         cookies module fails to reject invalid cookies
-        https://bitbucket.org/cherrypy/cherrypy/issues/1405
+        https://github.com/cherrypy/cherrypy/issues/1405
         """
         cookies = sys.modules.get('http.cookies')
         _is_legal_key = getattr(cookies, '_is_legal_key', lambda x: False)
